package inf1010.pusle.chat;

import java.io.*;
import java.nio.charset.Charset;

/**
 * A utility class containing constants for the chat system.
 */
public class ChatUtilities {

    /** The default port number used by the chat programs. */
    public static final int DEFAULT_PORT = 9513;

    /** Number of bytes in a packet header. */
    public static final int HEADER_LENGTH = 2;

    /** The maximum length of data sent in a packet. */
    public static final int MAX_LENGTH = 255;

    /** The character set used by the chat system (UTF-8). */
    public static final Charset charset = Charset.forName("UTF-8");

    /** Packet indices. */
    public static final int TYPE_INDEX = 0, LENGTH_INDEX = 1, DATA_INDEX = 2;

    /** Packet types. */
    public static final byte
        LOGIN = 0x1, // Send username
        WELCOME = 0x1, // Acknowledge username
        USER_LIST = 0x2, // List of who is logged into the chat
        NEW_USER = 0x3, // New user joined the chat
        USER_LEFT = 0x4, // User left the chat
        DISCONNECT = 0x5, // Disconnecting (closing socket)
        MESSAGE_FROM = 0x10, // User who sent the next message
        MESSAGE = 0x11, // Content of a chat message
        INVALID_USERNAME = (byte)0x80, // Username is in use already
        INVALID_PACKET = (byte)0x90; // Received an invalid packet

    /**
     * Sends data on an output stream.
     *
     * @param type the type of message to send to the server.
     * @param data the data to send with the message.
     * @param out the output stream to send data to.
     * @throws IOException if anything goes wrong with sending the data.
     * @return false if the data was too long.
     */
    public static boolean sendMessage(byte type, String data,
                             OutputStream out) throws IOException {
        byte[] bdata = data == null ? new byte[0] : data.getBytes(charset);

        /* TODO: Send header + data. */
        
        return true;
    }

    /**
     * Receives a message from an input stream.
     *
     * @param in the input stream to receive the message from.
     * @throws IOException if anything goes wrong with receiving the data.
     * @return a Message representing the packet.
     */
    public static Message recvMessage(InputStream in) throws IOException {

        /* TODO: Receive header + data. */
        
        return new Message(type, new String(data, charset));
    }
}