package inf1010.pusle.chat.client;

import inf1010.pusle.chat.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * A simple chat client for Puslegruppa.
 */
public class ChatClient implements Runnable, ChatListener {

    /** The port used by default. */
    private static final int DEFAULT_PORT = 9513;

    /** The socket connecting this client to the server. */
    private Socket sock;

    /** The socket's input stream. */
    private InputStream sin;
    /** The socket's output stream. */
    private OutputStream sout;

    /** The listener that handles chat messages. */
    LinkedList<ChatListener> listeners = new LinkedList<ChatListener>();

    /**
     * Entry point.
     *
     * @param args command line args: <ul>
     *             <li>server</li>
     *             <li>port</li>
     *             <li>username</li></ul>
     */
    public static void main(String[] args) {
        if (args.length < 2) {
            System.out.println("Invalid arguments.");
            System.out.println("Arguments: server (port) username");
            System.exit(1);
        }

        int arg = 0;
        String server = args[arg++];
        int port = DEFAULT_PORT;
        if (args.length > 2) {
            try {
                port = Integer.parseInt(args[arg++]);
            } catch (NumberFormatException nfe) {
                System.err.println("Second argument should be a port number.");
                System.err.println(nfe.getMessage());
                System.exit(1);
            }
        }
        String user = args[arg++];
        /*final*/ ChatClient cc = new ChatClient(server, port, user);
        /*
          new Thread() {
          Scanner scan = new Scanner(System.in);
          public void run() {
          while(true) {
          String s = scan.nextLine();
          if (!s.isEmpty()) {
          cc.sendMessage(s);
          }
          }
          }
          }.start();
        */
        new ChatFrame(cc);
        cc.run();
    }

    /**
     * Handles incoming packets not handled by other ChatListeners.
     *
     * @param m the packet to handle.
     * @throws IOException if sending error messages to the user fails.
     * @return true if the message was handled.
     */
    public boolean handleMessage(Message m) throws IOException {
        switch (m.getType()) {
        case ChatUtilities.MESSAGE_FROM:
            /* TODO: Handle message type. */
            break;
        case ChatUtilities.MESSAGE:
            /* TODO: Handle message type. */
            break;
        case ChatUtilities.DISCONNECT:
            /* TODO: Handle message type. */
            break;
        case ChatUtilities.USER_LIST:
            /* TODO: Handle message type. */
            break;
        case ChatUtilities.NEW_USER:
            /* TODO: Handle message type. */
            break;
        case ChatUtilities.USER_LEFT:
            /* TODO: Handle message type. */
            break;
        case ChatUtilities.INVALID_USERNAME:
            /* TODO: Handle message type. */
            break;
        case ChatUtilities.INVALID_PACKET:
            /* TODO: Handle message type. */
            break;
        default:
            break;
        }
        return true;
    }

    /**
     * Sends a chat message to the server.
     *
     * @param msg the message to send to the server.
     */
    public void sendMessage(String msg) {
        try {
            ChatUtilities.sendMessage(ChatUtilities.MESSAGE, msg, sout);
        } catch (IOException ioe) {
            System.err.println("Could not send message to server:");
            System.err.println(ioe.getMessage());
        }
    }

    /**
     * Create a ChatClient and connect it to a server.
     *
     * @param server the server to connect to.
     * @param port the port number to use.
     * @param user the username to use in the chat.
     */
    public ChatClient(String server, int port, String user) {

        addChatListener(this);

        /* TODO: Create socket and initialize input/output streams.
           TODO: Send LOGIN message to server. */

        // Handle shutdowns "gracefully"
        Runtime.getRuntime().addShutdownHook(new Thread() {
                public void run() {
                    try {
                        ChatUtilities.sendMessage(ChatUtilities.DISCONNECT,
                                                  null, sout);
                        sock.close();
                    } catch(IOException e) {
                        System.err.println("Could not close connection.");
                        System.err.println(e.getMessage());
                    }
                }
            });
    }

    /**
     * Changes this client's ChatListener.
     *
     * @param listener the new ChatListener to handle received messages.
     */
    public void addChatListener(ChatListener listener) {
        listeners.addFirst(listener);
    }

    /**
     * Listen to the socket until the socket gets closed.
     */
    public void run() {

        /* TODO: Exit if end of stream (message is null).
           TODO: */
        while (/* we are still connected */) {
            Message m = ChatUtilities.recvMessage(sin);

            for (ChatListener listener : listeners) {
                if (listener.handleMessage(m)) {
                    break;
                }
            }
        }
    }
}